/* hpfender Ball - komplettes Programm */
#include <math.h>      /* fr die Berechnung der Ballhhe */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#include "SDL.h"

struct {
SDL_AudioSpec spec;
Uint8   *sound;         /* Zeiger auf die Audiodaten */
Uint32   soundlen;      /* Lnge der Audiodaten */
int      soundpos;              /* Aktuelle Abspielposition */
} wave;

/* folgende Funktion wird aufgerufen, wenn die Audiohardware weitere Daten
bentigt */
void fillerup(void *unused, Uint8 *stream, int len)
{
Uint8 *waveptr;
int    waveleft;

/* Zeiger setzten */
waveptr = wave.sound + wave.soundpos;
waveleft = wave.soundlen - wave.soundpos;

/* Abspielen */
while ( waveleft <= len ) {
SDL_MixAudio(stream, waveptr, waveleft, SDL_MIX_MAXVOLUME);
stream += waveleft;
len -= waveleft;
waveptr = wave.sound;
waveleft = wave.soundlen;
wave.soundpos = 0;
}
SDL_MixAudio(stream, waveptr, len, SDL_MIX_MAXVOLUME);
wave.soundpos += len;
}

main (int argc, char **argv)
{
SDL_Surface *screen;
SDL_Event event;
SDL_Surface *image;
SDL_Surface *background;
SDL_Rect dest;
int ncolors, i;
SDL_Color *colors;

/* Initialisiere den Video-und Audioteil der SDL */
if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0 ) {
fprintf(stderr, "Kann SDL nicht initialisieren: %s\n", SDL_GetError());
return 1;
}

/* beim Beenden automatisch aufraeumen */
atexit(SDL_Quit);

/* oeffne ein Fenster der groesse 320 x 200 */
screen=SDL_SetVideoMode(320, 200, 8, SDL_SWSURFACE|SDL_HWPALETTE);
if ( screen == NULL ) {
fprintf(stderr, "Kann Videomodus nicht setzten: %s\n", SDL_GetError());
return 1;
}

/* BMP-Datei in ein neues Surface laden: */
image = SDL_LoadBMP("ball.bmp");
if ( image == NULL ) {
fprintf(stderr, "Konnte Bitmap nicht laden: %s\n", SDL_GetError());
return;
}

/* Farbpalette setzten
* SDL_SetColors() zeigt bei Hi- oder Truecolor-Modi keine Wirkung.
* falls die Farbpalette schon einmal zuvor gesetzt wurde, knnen
* die folgenden Anweisungen auch bersprungen werden
*/
if ( image->format->palette ) {
ncolors = image->format->palette->ncolors;
colors  = (SDL_Color *)malloc(ncolors*sizeof(SDL_Color));
memcpy(colors, image->format->palette->colors, ncolors);
} else{
int r, g, b;

/* Generiere allgemeine 256-Farbpalette,
da die Bitmap keine eigene Farbpalette mitbringt */
ncolors = 256;
colors  = (SDL_Color *)malloc(ncolors*sizeof(SDL_Color));

/* Setze einen 3,3,2 Farbwrfel */
for ( r=0; r<8; ++r ) {
for ( g=0; g<8; ++g ) {
for ( b=0; b<4; ++b ) {
i = ((r<<5)|(g<<2)|b);
colors[i].r = r<<5;
colors[i].g = g<<5;
colors[i].b = b<<6;
}
}
}
}/*else*/

/* neues background-surface anlegen */
/* dabei werden alle Einstellungen von der Bitmap bernommen */
background=SDL_AllocSurface(image->flags, image->w, image->h,
image->format->BitsPerPixel, image->format->Rmask,
image->format->Gmask, image->format->Bmask, 0);
if (background==NULL){
fprintf(stderr, "Konnte background-surface nicht anlegen: %s\n",
SDL_GetError());
return;
}

/* lade die Audiodatei */
SDL_LoadWAV("ball.wav", &wave.spec, &wave.sound, &wave.soundlen);
wave.spec.callback=fillerup;

SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);

/* Farbpalette setzen */
SDL_SetColors(screen, colors, 0, ncolors);
free(colors);

/* Hauptschleife */
while (SDL_WaitEvent(&event)>=0)
{
/* hier werden die Ereignisse behandelt */
switch(event.type){
case SDL_QUIT:{
/* Surfaces wieder frei geben */
SDL_FreeSurface(background);
SDL_FreeSurface(image);

/* Audiospeicher wieder freigeben */
SDL_FreeWAV(wave.sound);

exit(0);
}
case SDL_KEYDOWN:{

Uint8 *keys;

/* berprfen, ob es die Leertaste war */
keys=SDL_GetKeyState(NULL);

if(keys[SDLK_SPACE]==SDL_PRESSED){
double t;

/* ffne Audioausgabegert im gewnschten Format */
if( SDL_OpenAudio(&wave.spec, NULL) < 0 ) {
fprintf(stderr, "Kann Audio nicht ausgeben: %s\n",
SDL_GetError());
return(-1);
}

/* Pause aufheben, wodurch Wiedergabe gestartet wird */
SDL_PauseAudio(0);

/* sichere Screen in background */
SDL_BlitSurface(screen, &dest, background, NULL);
for (t=0; t<6*PI; t=t+(0.01)){
dest.x = 50;

/* hier wird die aktuelle Hhe berechnet */
dest.y = 100-(100*exp((-t)*0.3)*fabs(cos(t)));
dest.w = image->w;
dest.h = image->h;

/* kopiere Ball an seine Position */
SDL_BlitSurface(image, NULL, screen, &dest);
SDL_Flip(screen);

/* jetzt Hintergrund wieder zurckholen */
SDL_BlitSurface(background, NULL, screen, &dest);
} /*for*/

/* Animation beendet => Sound beenden */
SDL_CloseAudio();                                
}/*if*/
}/* case */
}/*switch*/
}/* do*/
return 0;
}
